function [ U, opts ] = hwscrt( f_func, a_bc_func, b_bc_func, c_bc_func, d_bc_func, varargin )
% HWSCRT - Solve the Helmholtz Equation in Cartesian coordinates using HWSCRT of FISHPACK.
% 
% The types of PDE's that can be solved by this routine take the following form
% 
% u_xx + u_yy + \lambda u = f(x,y)
% 
% With u a scalar unknown function u_xx and u_yy represent the two spatial derivatives
% with respect to x and y.  For either the x or y coordinate direction boundary conditions 
% take several forms:
% 
% - Periodic
% - Dirchlet on both faces
% - Neumann on both faces
% - Mixed (Dirchlet on one face with Neumann on the other)
% 
% The user of this routine must at minimum specify the following 
% octave functions:
% 
% A function to evaluate the right hand side f(x,y) (can be zero)
% A function to evaluate the right   hand boundary conditions (at x=A)
% " """""""" "" """""""" """ left    """" """""""" """""""""" (at x=B)
% " """""""" "" """""""" """ bottom  """" """""""" """""""""" (at y=C)
% " """""""" "" """""""" """ top     """" """""""" """""""""" (at y=D)
% 
% Written by:
% -- 
% John L. Weatherwax                2006-02-12
% 
% email: wax@alum.mit.edu
% 
% Please send comments and especially bug reports to the
% above email address.
% 
%-----

if( nargin < 5 ) error( 'Not enough input arguments' ); end

% Default hwscrt input arguments: 
%  

% The left end of the computational domain:
% 
A = -1;

% The right end of the computational domain:
% 
B = +1; 

% The number of panels into which the interval (A,B) is subdivided:
% 
M = 30; 

% The type of boundary conditions at x=A and X=B:
%
MBDCND='periodic';

% The bottom end of the computational domain:
% 
C = -1;

% The top end of the computational domain:
% 
D = +1; 

% The number of panels into which the interval (C,D) is subdivided:
% 
N = 30; 

% The type of boundary conditions at y=C and y=D:
%
NBDCND='periodic';

% The value of lambda in the the Helmholtz equation:
%
elmbda = 1.0; 

% Override default arguments with "varargin" inputs (if any):
%
i=1; 
while i < length(varargin)
  arg = varargin{i};
  switch lower(arg)
   case 'a'
    A = varargin{i+1};
   case 'b'
    B = varargin{i+1};
   case 'm'
    M = varargin{i+1}; 
   case { 'xbc', 'xboundaryconditions' } 
    MBDCND = varargin{i+1};
   case 'c'
    C = varargin{i+1};
   case 'd'
    D = varargin{i+1};
   case 'n'
    N = varargin{i+1}; 
   case { 'ybc', 'yboundaryconditions' } 
    NBDCND = varargin{i+1};
   case { 'lambda', 'elmbda' }
    elmbda = varargin{i+1};
   otherwise
    error( [ 'Unknown input: ', arg ] ); 
  end
  i=i+2;
end

% Create grids for possible evaluation of input functions:
% 
xbkpt = linspace( A, B, M ); 
ybkpt = linspace( C, D, N ); 

% Input argument error checking and translation:
%
switch lower(MBDCND)
 case {'p','periodic','period'}
  MBDCND=0; 
 case {'d','dirchlet','dirc'}
  MBDCND=1; 
 case {'dn','dirchletneumann','dirneu'}
  MBDCND=2; 
 case {'n','neumann'}
  MBDCND=3; 
 case {'nd','neumanndirchlet','neudir'}
  MBDCND=4;
 otherwise
  error('Unknown value of x-boundary conditions'); 
end

switch lower(NBDCND)
 case {'p','periodic','period'}
  NBDCND=0; 
 case {'d','dirchlet','dirc'}
  NBDCND=1; 
 case {'dn','dirchletneumann','dirneu'}
  NBDCND=2; 
 case {'n','neumann'}
  NBDCND=3; 
 case {'nd','neumanndirchlet','neudir'}
  NBDCND=4;
 otherwise
  error('Unknown value of y-boundary conditions'); 
end

% Create a vector for BDA: 
%
if( isvector( a_bc_func ) )          % BC's are given as a vector ...
  BDA = a_bc_func;
elseif( isempty( a_bc_func ) )       % BC's are given as empty ... 
  BDA = 0;                           
else                                 % BC's are given as a function ...
  BDA = feval( a_bc_func, ybkpt ); 
end

% Create a vector for BDB: 
%
if( isvector( b_bc_func ) )          % BC's are given as a vector ...
  BDB = b_bc_func;
elseif( isempty( b_bc_func ) )       % BC's are given as empty ... 
  BDB = 0;                           
else                                 % BC's are given as a function ...
  BDB = feval( b_bc_func, ybkpt ); 
end

% Create a vector for BDC: 
%
if( isvector( c_bc_func ) )          % BC's are given as a vector ...
  BDC = c_bc_func;
elseif( isempty( c_bc_func ) )       % BC's are given as empty ... 
  BDC = 0;                           
else                                 % BC's are given as a function ...
  BDC = feval( c_bc_func, ybkpt ); 
end

% Create a vector for BDD: 
%
if( isvector( d_bc_func ) )          % BC's are given as a vector ...
  BDD = d_bc_func;
elseif( isempty( d_bc_func ) )       % BC's are given as empty ... 
  BDD = 0;                           
else                                 % BC's are given as a function ...
  BDD = feval( d_bc_func, ybkpt ); 
end

% Create a matrix F:
%
if( ismatrix( f_func ) )         % the right hand side is given as a matrix ...
  F = f_func;
elseif( isempty( f_func ) )      % the right hand side is empty ...
  F = 0; 
else                             % the right hand side is given as a function ...
  F = feval( f_func, xbkpt, ybkpt ); 
end

% Call the octave *.oct file that calls the FORTRAN routine hwscrt:
%
U = hwscrtg( A, B, M, MBDCND, BDA, BDB, C, D, N, NBDCND, BDC, BDD, elmbda, F );

% Collect all inputs together into a structure:
%
opts.A =          A; 
opts.B =          B; 
opts.M =          M; 
opts.MBDCND =     MBDCND; 
opts.C =          C; 
opts.D =          D; 
opts.N =          N; 
opts.NBDCND =     NBDCND; 
opts.elmbda =     elmbda; 
opts.xbkpt =      xbkpt; 
opts.ybkpt =      ybkpt; 

